@@ -12,7 +12,7 @@ pub fn main() {
         .build(&event_loop)
         .unwrap();
 
-    #[cfg(feature = "web-sys")]
+    #[cfg(target_arch = "wasm32")]
     {
         use winit::platform::web::WindowExtWebSys;
 
@@ -26,28 +26,12 @@ pub fn main() {
             .expect("Append canvas to HTML body");
     }
 
-    #[cfg(feature = "stdweb")]
-    {
-        use std_web::web::INode;
-        use winit::platform::web::WindowExtStdweb;
-
-        let canvas = window.canvas();
-
-        let document = std_web::web::document();
-        let body: std_web::web::Node = document.body().expect("Get HTML body").into();
-
-        body.append_child(&canvas);
-    }
-
     event_loop.run(move |event, _, control_flow| {
         *control_flow = ControlFlow::Wait;
 
-        #[cfg(feature = "web-sys")]
+        #[cfg(target_arch = "wasm32")]
         log::debug!("{:?}", event);
 
-        #[cfg(feature = "stdweb")]
-        std_web::console!(log, "%s", format!("{:?}", event));
-
         match event {
             Event::WindowEvent {
                 event: WindowEvent::CloseRequested,
@@ -61,7 +45,7 @@ pub fn main() {
     });
 }
 
-#[cfg(feature = "web-sys")]
+#[cfg(target_arch = "wasm32")]
 mod wasm {
     use wasm_bindgen::prelude::*;
 
