@@ -15,11 +15,12 @@ use std::{
 use cocoa::{
     appkit::{NSApp, NSApplication, NSWindow},
     base::{id, nil},
-    foundation::{NSAutoreleasePool, NSSize},
+    foundation::NSSize,
+};
+use objc::{
+    rc::autoreleasepool,
+    runtime::{Object, YES},
 };
-use objc::runtime::YES;
-
-use objc::runtime::Object;
 
 use crate::{
     dpi::LogicalSize,
@@ -306,7 +307,9 @@ impl AppState {
         let panic_info = panic_info
             .upgrade()
             .expect("The panic info must exist here. This failure indicates a developer error.");
-        if panic_info.is_panicking() || !HANDLER.is_ready() {
+
+        // Return when in callback due to https://github.com/rust-windowing/winit/issues/1779
+        if panic_info.is_panicking() || !HANDLER.is_ready() || HANDLER.get_in_callback() {
             return;
         }
         let start = HANDLER.get_start_time().unwrap();
@@ -370,24 +373,25 @@ impl AppState {
         let panic_info = panic_info
             .upgrade()
             .expect("The panic info must exist here. This failure indicates a developer error.");
-        if panic_info.is_panicking() || !HANDLER.is_ready() {
+
+        // Return when in callback due to https://github.com/rust-windowing/winit/issues/1779
+        if panic_info.is_panicking() || !HANDLER.is_ready() || HANDLER.get_in_callback() {
             return;
         }
-        if !HANDLER.get_in_callback() {
-            HANDLER.set_in_callback(true);
-            HANDLER.handle_user_events();
-            for event in HANDLER.take_events() {
-                HANDLER.handle_nonuser_event(event);
-            }
-            HANDLER.handle_nonuser_event(EventWrapper::StaticEvent(Event::MainEventsCleared));
-            for window_id in HANDLER.should_redraw() {
-                HANDLER.handle_nonuser_event(EventWrapper::StaticEvent(Event::RedrawRequested(
-                    window_id,
-                )));
-            }
-            HANDLER.handle_nonuser_event(EventWrapper::StaticEvent(Event::RedrawEventsCleared));
-            HANDLER.set_in_callback(false);
+
+        HANDLER.set_in_callback(true);
+        HANDLER.handle_user_events();
+        for event in HANDLER.take_events() {
+            HANDLER.handle_nonuser_event(event);
         }
+        HANDLER.handle_nonuser_event(EventWrapper::StaticEvent(Event::MainEventsCleared));
+        for window_id in HANDLER.should_redraw() {
+            HANDLER
+                .handle_nonuser_event(EventWrapper::StaticEvent(Event::RedrawRequested(window_id)));
+        }
+        HANDLER.handle_nonuser_event(EventWrapper::StaticEvent(Event::RedrawEventsCleared));
+        HANDLER.set_in_callback(false);
+
         if HANDLER.should_exit() {
             unsafe {
                 let app: id = NSApp();
@@ -403,16 +407,16 @@ impl AppState {
                 };
 
                 let dialog_is_closing = HANDLER.dialog_is_closing.load(Ordering::SeqCst);
-                let pool = NSAutoreleasePool::new(nil);
-                if !INTERRUPT_EVENT_LOOP_EXIT.load(Ordering::SeqCst)
-                    && !dialog_open
-                    && !dialog_is_closing
-                {
-                    let () = msg_send![app, stop: nil];
-                    // To stop event loop immediately, we need to post some event here.
-                    post_dummy_event(app);
-                }
-                pool.drain();
+                autoreleasepool(|| {
+                    if !INTERRUPT_EVENT_LOOP_EXIT.load(Ordering::SeqCst)
+                        && !dialog_open
+                        && !dialog_is_closing
+                    {
+                        let () = msg_send![app, stop: nil];
+                        // To stop event loop immediately, we need to post some event here.
+                        post_dummy_event(app);
+                    }
+                });
 
                 if window_count > 0 {
                     let window: id = msg_send![windows, objectAtIndex:0];
